/*
 * JORAM: Java(TM) Open Reliable Asynchronous Messaging
 * Copyright (C) 2007 ScalAgent Distributed Technologies
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): ScalAgent Distributed Technologies
 * Contributor(s): 
 */
package org.objectweb.joram.kclient;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

public class TcpChannel extends Channel {
  class TcpMessage {
    long id;
    AbstractRequest msg;
  
    TcpMessage(long id, AbstractRequest msg) {
      this.id = id;
      this.msg = msg;
    }
  }

  private String user;
  private String pass;
  private String host;
  private int port;

// AF: TODO
//   Socket socket;
  InputStream sockin;
  OutputStream sockout;
  OutputXStream out;
  InputXStream in;

  // definition from ReliableTcpConnection
  int windowSize;
  long inputCounter;
  long outputCounter;
  int unackCounter;
  Vector pendingMessages;

  public TcpChannel(String user, String pass, String host, int port) {
    this.user = user;
    this.pass = pass;
    this.host = host;
    this.port = port;

    windowSize = 10;
    inputCounter = -1;
    outputCounter = 0;
    unackCounter = 0;
    pendingMessages = new Vector();

    key = -1;
    status = INIT;

// AF: TODO
//    socket = null;
    sockout = null;
    sockin = null;
    
    out = null;
    in = null;
  }

  public void connect() throws IOException {
    if (status != INIT) throw new IOException("Bad status");
    status = CONNECTING;

//     struct sockaddr_in addr;

//     sock = socket(AF_INET, SOCK_STREAM, 0);
//     if (sock == -1) throw ConnectException("Cannot create socket");

//     addr.sin_family = AF_INET;
//     addr.sin_port   = htons(port);

//     struct hostent *server = gethostbyname(host);
//     if (server == null) throw ConnectException("Host unknown");
//     bcopy((char *)server.h_addr, 
//           (char *)&addr.sin_addr.s_addr,
//           server.h_length);
//     if (::connect(sock, (sockaddr*) &addr, sizeof(addr)) == -1)
//       throw ConnectException("Cannot connect");

// AF: TODO
//    sockout = sock.getOutputStream();
//    sockin = sock.getInputStream();

    out = new OutputXStream();
    in = new InputXStream();

    out.writeString(user);
    out.writeString(pass);
    out.writeInt(key);

    if (key == -1) {
      // Open new connection
      out.writeInt(/* reconnectTimeout */ 0);

      out.writeTo(sockout);
      in.readFrom(sockin);

      int res = in.readInt();
      if (res > 0) {
        String info = in.readString();
        throw new IOException(info);
      }
      key = in.readInt();
    } else {
      // Reopen the connection
      out.writeTo(sockout);
      in.readFrom(sockin);

      int res = in.readInt();
      if (res > 0) {
        String info = in.readString();
        throw new IOException(info);
      }
    }

    TcpMessage pendingMsg = null;
    for (int i = 0; i < pendingMessages.size(); i++) {
      pendingMsg = (TcpMessage) pendingMessages.elementAt(i);
      doSend(pendingMsg.id, inputCounter, pendingMsg.msg);
    }

    status = CONNECTED;
  }

  public void send(AbstractRequest request) throws IOException {
    if (status != CONNECTED) throw new IOException("Connection closed");

    try {      
      doSend(outputCounter, inputCounter, request);
      pendingMessages.addElement(new TcpMessage(outputCounter, request));
      outputCounter++;
    } catch (IOException exc) {
      close();
      throw exc;
    }
  }

  void doSend(long id, long counter, AbstractRequest request) throws IOException {
    if (status != CONNECTED) throw new IOException("Bad status");

    out.writeLong(id);
    out.writeLong(counter);
    AbstractMessage.write(request, out);
    out.writeTo(sockout);
  }

  public AbstractReply receive() throws Exception {
    if (status != CONNECTED) throw new IOException("Bad status");

    while (true) {
      in.readFrom(sockin);

      long msgid, ackid;

      msgid = in.readLong();
      ackid = in.readLong();
      AbstractReply reply = (AbstractReply) AbstractMessage.read(in);

      while (pendingMessages.size() > 0) {
        TcpMessage pendingMsg = (TcpMessage) pendingMessages.elementAt(0);
        if (ackid < pendingMsg.id) {
          // It's an old acknowledge
          break;
        } else {
          pendingMessages.removeElementAt(0);
        }
      }

      if (reply != null) {
        if (unackCounter < windowSize) {
          unackCounter++;
        } else {
          // TODO
/*           AckTimerTask ackTimertask = new AckTimerTask(); */
/*           timer.schedule(ackTimertask, 0); */
        }
        if (msgid > inputCounter) {
          inputCounter = msgid;
          return reply;
        }
      }
    }
  }

  public void close() throws IOException {
    sockin.close();
    sockout.close();
// AF: TODO
//    close(sock);
  }
}
